import asyncio
from collections import namedtuple
from struct import unpack
import openhab

# TODO: noch optimieren: nur bei Änderung senden, außerdem dieses Programm als daemon laufen lassen

# Adresse der Wetterstation
PWS_IP = '192.168.0.34'
PWS_PORT = 45000

# Struktur zum Entschlüsslen der Daten
Response_Aktuell = namedtuple('Response_Aktuell', 'startstring    len_struct num_struct num_Temp_In wsTemp_In  num_Temp_Out wsTemp_Out  num_Dewpoint_Out wsDewpoint_Out  num_windchill wsWindchill  num_heatindex wsHeatindex  num_Humidity_In wsHumidity_In num_aussen_feuchte wsHumidity_Out num_abs_Pressure abs_Pressure  num_Pressure wsPressure  num_Wind_Degree wsWind_Degree  num_Wind_Speed wsWind_Speed  num_Wind_Gust wsWind_Gust  num_Rain_Rate wsRain_Rate    num_Rain_Day wsRain_Day    num_Rain_Week wsRain_Week    num_Rain_Month wsRain_Month    num_Rain_Year wsRain_Year    num_Rain_Total wsRain_Total    num_Ambient_Light wsAmbient_Light    num_Ambient_Light_ADC wsAmbient_Light_ADC  num_UV_Index wsUV_Index timestamp checksum')

# in diesem Dict sind alle IDs enthalten, die an zum Test an der richtigen Stelle liegen müssen
# wenn dieser Test erfüllt ist, dann kann man sicher sein, dass die struktur mit den restlichen werten passt
Dict_Aktuell_Test = {'startstring': 0xFFFF0B00,
                     'num_struct': 0x04,
                     'num_Temp_In': 0x01,
                     'num_Temp_Out': 0x02,
                     'num_Dewpoint_Out': 0x03,
                     'num_windchill': 0x04,
                     'num_heatindex': 0x05,
                     'num_Humidity_In': 0x06,
                     'num_aussen_feuchte': 0x07,
                     'num_abs_Pressure': 0x08,
                     'num_Pressure': 0x09,
                     'num_Wind_Degree': 0x0A,
                     'num_Wind_Speed': 0x0B,
                     'num_Wind_Gust': 0x0C,
                     'num_Rain_Rate': 0x0E,
                     'num_Rain_Day': 0x10,
                     'num_Rain_Week': 0x11,
                     'num_Rain_Month': 0x12,
                     'num_Rain_Year': 0x13,
                     'num_Rain_Total': 0x14,
                     'num_Ambient_Light': 0x15,
                     'num_Ambient_Light_ADC': 0x16,
                     'num_UV_Index': 0x17
                     }

# in dieser Dict sind alle Werte mit Wetterdaten und Umrechnungsfaktor enthalten
Dict_Aktuell_Umrechnung = {'wsTemp_In': 0.1,
                           'wsTemp_Out': 0.1,
                           'wsDewpoint_Out': 0.1,
                           'wsWindchill': 0.1,
                           'wsHeatindex': 0.1,
                           'wsHumidity_In': 1,
                           'wsHumidity_Out': 1,
                           'wsPressure': 0.1,
                           'wsWind_Degree': 1,
                           'wsWind_Speed': 0.36,
                           'wsWind_Gust': 0.36,
                           'wsRain_Rate': 0.1,
                           'wsRain_Day': 0.1,
                           'wsRain_Week': 0.1,
                           'wsRain_Month': 0.1,
                           'wsRain_Year': 0.1,
                           'wsRain_Total': 0.1,
                           'wsAmbient_Light': 0.1,
                           'wsAmbient_Light_ADC': 1,
                           'wsUV_Index': 1
                           }


async def aktuelle_daten_laden():
    # zum Abholen der Daten muss eine bestimmte Meldung an die WS geschickt werden
    REQUEST = bytes.fromhex("ffff0b0006040419")
    # die Länge der Antwort ist immer Fest
    RESPONSE_LEN = 82
    # string zum entpacken der Bytefolge in eigene typen
    RESPONSE_STRUCT = "!LBBBhBhBhBhBhBBBBBHBHBHBHBHBLBLBLBLBLBLBLBHBBBB"

    reader, writer = await asyncio.open_connection(PWS_IP, PWS_PORT)

    writer.write(REQUEST)
    data = await reader.read(RESPONSE_LEN)

    writer.close()

    # Antwort decodieren
    werte = Response_Aktuell._make(unpack(RESPONSE_STRUCT, data))

    # Antwort prüfen
    # 1) passt die Prüfsumme?
    if werte.checksum == sum([byte for byte in data[2:len(data)-1]]) & 0xFF:
        # 2) sind alle IDs an den korrekten Stellen?
        werte_dict = werte._asdict()
        check = True
        for k, v in Dict_Aktuell_Test.items():
            if werte_dict[k] != v:
                #print(f"Für den Wert {k!r} wurde {v!r} erwartet, stattdessen wurde {werte_dict[k]!r} gefunden")
                check = False

        if check:
            for k, v in Dict_Aktuell_Umrechnung.items():
                messwert = werte_dict[k]*v
                #print(f"{k} = {messwert}")
                openhab.send_value(k, messwert)

asyncio.run(aktuelle_daten_laden())
